<?php

namespace Drupal\Tests\genpass\Functional;

use Drupal\genpass\GenpassInterface;
use Drupal\Tests\BrowserTestBase;

/**
 * Tests functionality of the Generate Password module.
 *
 * @group genpass
 */
class GenpassTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $strictConfigSchema = TRUE;

  /**
   * Modules to install.
   *
   * @var array
   */
  protected static $modules = [
    'user',
    'genpass',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with "administer account settings".
   *
   * And "administer users" permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $webUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp():void {
    parent::setUp();

    $permissions = [
      'view the administration theme',
      'administer account settings',
      'administer users',
    ];

    $this->webUser = $this->drupalCreateUser($permissions);
    $this->drupalLogin($this->webUser);
  }

  /**
   * Test that all the changes to the accounts settings form are reflected.
   */
  public function testGenpassAdminFormChanges() {
    $this->drupalGet('admin/config/people/accounts');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Account settings');
    $this->assertSession()->pageTextContains('Generate Password - User Account Registration');
    $this->assertSession()->pageTextContains('User password entry');
    $this->assertSession()->pageTextContains('Admin password entry');
    $this->assertSession()->pageTextContains('Generated password length');
    $this->assertSession()->pageTextContains('Generated password display');
  }

  /**
   * Test Generate Password configs and create users by admin.
   *
   * @param array $config
   *   Configuration values to set in form prior to submission.
   * @param array $expects
   *   Array of expected and not expected message text to check for.
   *
   * @dataProvider configTestsDataProvider
   */
  public function testGenpassConfigs(array $config, array $expects) {
    // Load the accounts page and confirm it works.
    $this->drupalGet('admin/config/people/accounts');
    $this->assertSession()->statusCodeEquals(200);

    // Set the configuration values into an edit form submission.
    $this->submitForm($config, 'Save configuration');

    // Check for text that both exists and not.
    foreach (($expects['text'] ?? []) as $text) {
      $this->assertSession()->pageTextContains($text);
    }
    foreach (($expects['nottext'] ?? []) as $text) {
      $this->assertSession()->pageTextNotContains($text);
    }
  }

  /**
   * Data provider for testGenpassConfigs.
   *
   * @return \Generator
   *   An array of configuration options to set, and expected page text output.
   */
  public static function configTestsDataProvider(): \Generator {

    // Test to confirm that genpass_mode 0 will error when email_verify is on.
    yield [
      [
        'user_email_verification' => '1',
        'genpass_mode' => GenpassInterface::PASSWORD_REQUIRED,
      ],
      [
        'text' => ['User password entry option Users must enter a password on registration is not available when email verification is enabled.'],
        'nottext' => ['The configuration options have been saved.'],
      ],
    ];

    // Test to confirm that genpass_mode 1 will error when email_verify is on.
    yield [
      [
        'user_email_verification' => '1',
        'genpass_mode' => GenpassInterface::PASSWORD_OPTIONAL,
      ],
      [
        'text' => ['User password entry option Users may enter a password on registration is not available when email verification is enabled.'],
        'nottext' => ['The configuration options have been saved.'],
      ],
    ];

    // Test to confirm that email_verify turned off will allow genpass_mode 0.
    yield [
      [
        'user_email_verification' => '0',
        'genpass_mode' => GenpassInterface::PASSWORD_REQUIRED,
      ],
      [
        'text' => ['The configuration options have been saved.'],
      ],
    ];

    // Test the default options work.
    yield [
      [
        'user_email_verification' => '1',
        'genpass_mode' => GenpassInterface::PASSWORD_RESTRICTED,
      ],
      [
        'text' => ['The configuration options have been saved.'],
      ],
    ];
  }

  /**
   * Create a user via admin form and check output says password created.
   */
  public function testCreateUserByAdmin() {

    // Create the test_authenticated user.
    $this->drupalGet('admin/people/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Add user');
    $edit = [
      'mail' => 'authenticated.test@drupal.org',
      'name' => 'test_authenticated',
    ];
    $this->submitForm($edit, 'Create new account');

    $this->assertSession()->pageTextContains('Since you did not provide a password, it was generated automatically for this account.');
    $this->assertSession()->pageTextContains('Created a new user account for test_authenticated. No email has been sent.');
  }

  /**
   * Test Generate Password hide password field functionality.
   */
  public function testGenpassHidePasswordField() {

    // Allow admins to set passwords.
    $this->drupalGet('admin/config/people/accounts');
    $this->assertSession()->pageTextContains('Admin password entry');

    $this->getSession()->getPage()->selectFieldOption('genpass_admin_mode', '1');
    $this->getSession()->getPage()->pressButton('Save configuration');

    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Create the test_authenticated user.
    $this->drupalGet('admin/people/create');
    $this->assertSession()->pageTextContains('Provide a password for the new account in both fields.');

    // Disallow admins to set passwords.
    $this->drupalGet('admin/config/people/accounts');
    $this->getSession()->getPage()->selectFieldOption('genpass_admin_mode', '2');
    $this->getSession()->getPage()->pressButton('Save configuration');
    $this->assertSession()->pageTextContains('The configuration options have been saved.');

    // Create the test_authenticated user.
    $this->drupalGet('admin/people/create');
    $this->assertSession()->pageTextNotContains('Provide a password for the new account in both fields.');
    $this->getSession()->getPage()->fillField('mail', 'authenticated.test@drupal.org');
    $this->getSession()->getPage()->fillField('Username', 'test_authenticated');
    $this->getSession()->getPage()->pressButton('Create new account');
    $this->assertSession()->pageTextContains('Since you did not provide a password, it was generated automatically for this account.');
    $this->assertSession()->pageTextContains('Created a new user account for test_authenticated. No email has been sent.');
  }

}
